@@ -576,13 +576,12 @@ static inline int32_t maxbitsize_generators(bs_t *bs){
     int32_t ngens = bs->ld;
     for(int32_t i = 0; i < ngens; i++){
         int32_t len = bs->hm[i][LENGTH];
-        mpz_t *cfs = bs->cf_qq[i];
-        for(int32_t i = 0; i < len; i++){
+        mpz_t *cfs = bs->cf_qq[bs->hm[i][COEFFS]];
+        for(int32_t j = 0; j < len; j++){
+            max = MAX(max,
+                    mpz_sizeinbase(cfs[j], 2)
+                    );
         }
-        max = MAX(max, 
-                mpz_sizeinbase(cfs[i], 2)
-                );
-
     }
     return max;
 }
@@ -1148,7 +1147,8 @@ gb_modpoly_t *core_groebner_qq(
         mstrace_t msd,
         md_t *st,
         int32_t *errp, 
-        const len_t fc
+        const len_t fc, 
+        const int print_gb
         )
 {
   double st0 = realtime();
@@ -1159,7 +1159,6 @@ gb_modpoly_t *core_groebner_qq(
   msd->bht   = msd->bs_qq->ht;
 
   int32_t info_level = st->info_level;
-  int32_t print_gb = st->print_gb;
   int64_t dquot = 0;
   int64_t *dquot_ptr = &dquot;
   int32_t truncate_lifting = st->truncate_lifting;
@@ -1296,7 +1295,7 @@ gb_modpoly_t *core_groebner_qq(
       free_rrec_data(recdata2);
 
       fprintf(stderr, "Something went wrong in the learning phase, msolve restarts.");
-      return core_groebner_qq(modgbsp, bs, msd, st, errp, fc); 
+      return core_groebner_qq(modgbsp, bs, msd, st, errp, fc, print_gb); 
     }
     /* duplicate data for multi-threaded multi-mod computation */
     duplicate_data_mthread_gbtrace(st->nthrds, msd->bs_qq, st, msd->num_gb,
@@ -1422,7 +1421,7 @@ gb_modpoly_t *core_groebner_qq(
         free_rrec_data(recdata1);
         free_rrec_data(recdata2);
 
-        return core_groebner_qq(modgbsp, bs, msd, st, errp, fc); 
+        return core_groebner_qq(modgbsp, bs, msd, st, errp, fc, print_gb); 
 
       }
 
@@ -1454,6 +1453,9 @@ gb_modpoly_t *core_groebner_qq(
           break;
         }
       }
+      if(!bad && print_gb == 1){
+          apply = 0;
+      }
       if(dlift->lstart != lstart){
         if(info_level){
           fprintf(stdout, "<%.2f%%>", 100* (float)MIN((dlift->lstart + 1), (*modgbsp)->ld)/(*modgbsp)->ld);
@@ -1647,7 +1649,7 @@ gb_modpoly_t *groebner_qq(
   int err = 0;
 
   gb_modpoly_t *modgbsp = malloc(sizeof(gb_modpoly_t));
-  modgbsp = core_groebner_qq(modgbsp, bs, msd, md, &err, field_char);
+  modgbsp = core_groebner_qq(modgbsp, bs, msd, md, &err, field_char, print_gb);
   if (err) {
       printf("Problem with groebner_qq, stopped computation (%d).\n", err);
       exit(1);
@@ -1822,7 +1824,8 @@ int64_t export_groebner_qq(
     int err = 0;
 
     gb_modpoly_t *modgbsp = malloc(sizeof(gb_modpoly_t));
-    modgbsp = core_groebner_qq(modgbsp, bs, msd, md, &err, field_char);
+    modgbsp = core_groebner_qq(modgbsp, bs, msd, md, &err, field_char, 
+            2/* if set to 1, only the LM of the Gbs are correct */);
     if (err) {
         printf("Problem with groebner_qq, stopped computation.\n");
         exit(1);
