@@ -1,9 +1,9 @@
 #!/bin/bash
 
-file=bug-chgvar
+file=reals_dim0-chgvar
 
 $(pwd)/msolve -f input_files/$file.ms -o test/diff/$file.res \
-      -l 2 -t 1
+      -d 0 -l 2 -t 1
 if [ $? -gt 0 ]; then
     exit 1
 fi
@@ -14,7 +14,7 @@ if [ $? -gt 0 ]; then
 fi
 
 $(pwd)/msolve -f input_files/$file.ms -o test/diff/$file.res \
-      -l 2 -t 2
+      -d 0 -l 2 -t 2
 if [ $? -gt 0 ]; then
     exit 21
 fi
@@ -25,7 +25,7 @@ if [ $? -gt 0 ]; then
 fi
 
 $(pwd)/msolve -f input_files/$file.ms -o test/diff/$file.res \
-      -l 44 -t 1
+      -d 0 -l 44 -t 1
 if [ $? -gt 0 ]; then
     exit 41
 fi
@@ -36,7 +36,7 @@ if [ $? -gt 0 ]; then
 fi
 
 $(pwd)/msolve -f input_files/$file.ms -o test/diff/$file.res \
-      -l 44 -t 2
+      -d 0 -l 44 -t 2
 if [ $? -gt 0 ]; then
     exit 61
 fi
